#!/bin/bash

if [ "$(id -u)" != "0" ]; then
    echo "This script must be run as root" 1>&2
    exit 1
fi

die() { echo "ERROR: $*" 1>&2 ; exit 1; }

arch="$(uname -m)"
osname="$(uname)"

if ! [[ $arch = "x86_64" ]]; then
    die "This package can only be installed on Linux x86_64 system"
fi

if ! [[ $osname = "Linux" ]]; then
    die "This package can only be installed on Linux x86_64 system"
fi


src_dir="$(dirname $(readlink -f $0))"
dst_dir="/opt/cubebackup"

mkdir -p "${dst_dir}" \
&& mkdir -p "${dst_dir}/bin" \
&& mkdir -p "${dst_dir}/etc" \
&& mkdir -p "${dst_dir}/db" \
&& mkdir -p "${dst_dir}/locks" \
&& mkdir -p "${dst_dir}/log"

if ! [ $? -eq 0 ]; then
    echo "mkdir failed"
    exit 1
fi

if ! id -u cbuser > /dev/null 2>&1; then
    useradd -r -c "CubeBackup system user" cbuser || die "add user failed"
fi

chown cbuser "${dst_dir}/db"
chown cbuser "${dst_dir}/log"
chown cbuser "${dst_dir}/locks"

chmod a+x "${src_dir}/cbackup"
chmod a+x "${src_dir}/cbsrv"

if [ -f "${dst_dir}/bin/config.toml" ]; then
    mv "${dst_dir}/bin/config.toml" "${dst_dir}/etc/config.toml"
fi

web_port="80"
if [ ! -f "${dst_dir}/etc/config.toml" ]; then
    port_ok="yes"
    while true; do
        if ! [[ $web_port =~ ^[0-9]+$ ]] ; then
            echo "error: Not a number ${web_port}"
            port_ok="no"
        elif "${src_dir}/cbackup" isLocalPortOpen ${web_port} 2>/dev/null; then
            echo "Port ${web_port} is alreay in use, please assign another port for the web console."
            port_ok="no"
        else
            port_ok="yes"
        fi

        if [[ $port_ok = "no" ]]; then
            echo -n "Enter a new port and press [ENTER]:"
            read web_port </dev/tty
        else
            break
        fi
    done

    cp -f "${src_dir}/config.toml" "${dst_dir}/etc/config.toml"
    sed -i -e "s/WEBPORT/:${web_port}/g" "${dst_dir}/etc/config.toml"
else
    #find old version
    "${dst_dir}/bin/cbsrv" stop >/dev/null 2>&1
    "${dst_dir}/bin/cbsrv" uninstall >/dev/null 2>&1

    web_port="$(${src_dir}/cbackup printWebPort ${dst_dir}/etc/config.toml 2>/dev/null)"
fi

cp -f "${src_dir}/cbackup" "${dst_dir}/bin/cbackup" || die "copy cbackup failed"
cp -f "${src_dir}/cbsrv" "${dst_dir}/bin/cbsrv" || die "copy cbsrv failed"

if [ ! -f /.dockerenv ];then
    "${dst_dir}/bin/cbsrv" install || die "install service failed"
    "${dst_dir}/bin/cbsrv" start || die "start service failed"
fi

current_version="$(${dst_dir}/bin/cbackup version)"

echo "${current_version} was successfully installed."
echo "You can configure through the web console: http://<server ip>:${web_port}"
